@@ -11,11 +11,12 @@ from bach import DataFrame
 from bach.expression import Expression, quote_string, quote_identifier
 from bach.series import Series
 from bach.series import SeriesJson
-from bach.series.series_json import JsonPostgresAccessor
+from bach.series.series_json import JsonAccessor
 from bach.types import register_dtype
+from sql_models.util import is_postgres, DatabaseNotSupportedException, is_bigquery
 
 
-class ObjectivStack(JsonPostgresAccessor):
+class ObjectivStack(JsonAccessor):
     def get_from_context_with_type_series(self, type: str, key: str, dtype='string'):
         """
         .. _get_from_context_with_type_series:
@@ -27,6 +28,14 @@ class ObjectivStack(JsonPostgresAccessor):
         :param dtype: the dtype of the series to return.
         :returns: a series of type `dtype`
         """
+        dialect = self._series_object.engine.dialect
+        if is_postgres(dialect):
+            return self._postgres_get_from_context_with_type_series(type, key, dtype)
+        if is_bigquery(dialect):
+            return self._bigquery_get_from_context_with_type_series(type, key, dtype)
+        raise DatabaseNotSupportedException(dialect)
+
+    def _postgres_get_from_context_with_type_series(self, type: str, key: str, dtype='string'):
         dialect = self._series_object.engine.dialect
         expression_str = f'''
         jsonb_path_query_first({{}},
@@ -39,6 +48,21 @@ class ObjectivStack(JsonPostgresAccessor):
         )
         return self._series_object.copy_override_dtype(dtype).copy_override(expression=expression)
 
+    def _bigquery_get_from_context_with_type_series(self, type: str, key: str, dtype='string'):
+        select_ctx_expression = Expression.construct(
+            '''(
+              select first_value(ctx) over (order by pos)
+              from unnest(json_query_array({}, '$')) as ctx with offset as pos
+              where json_value(ctx, '$."_type"') = {}
+            )''',
+            self._series_object,
+            Expression.string_value(type)
+        )
+        ctx_series = self._series_object.copy_override(expression=select_ctx_expression)
+        as_str = dtype == 'string'
+        value_series = ctx_series.json.get_value(key=key, as_str=as_str)
+        return value_series.copy_override_dtype(dtype)
+
 
 @register_dtype(value_types=[], override_registered_types=True)
 class SeriesGlobalContexts(SeriesJson):
@@ -47,7 +71,6 @@ class SeriesGlobalContexts(SeriesJson):
     Contexts.
     """
     dtype = 'objectiv_global_context'
-    return_dtype = dtype
 
     class GlobalContexts(ObjectivStack):
         @property
@@ -136,7 +159,6 @@ class SeriesLocationStack(SeriesJson):
     Stack.
     """
     dtype = 'objectiv_location_stack'
-    return_dtype = dtype
 
     class LocationStack(ObjectivStack):
         @property
