@@ -4,18 +4,14 @@ Copyright 2021 Objectiv B.V.
 
 # Any import from modelhub initializes all the types, do not remove
 from modelhub import __version__
-from tests_modelhub.functional.modelhub.data_and_utils import get_objectiv_dataframe_test
+import pytest
+from tests_modelhub.data_and_utils.utils import get_objectiv_dataframe_test
 from tests.functional.bach.test_data_and_utils import assert_equals_data
 from uuid import UUID
 
-
-def test_get_objectiv_stack():
-    get_objectiv_dataframe_test()
-
-
 # map
-def test_is_first_session():
-    df, modelhub = get_objectiv_dataframe_test(time_aggregation='YYYY-MM-DD')
+def test_is_first_session(db_params):
+    df, modelhub = get_objectiv_dataframe_test(db_params, time_aggregation='%Y-%m-%d')
 
     s = modelhub.map.is_first_session(df)
 
@@ -36,7 +32,8 @@ def test_is_first_session():
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac311'), True],
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac312'), True]
         ],
-        order_by='event_id'
+        order_by='event_id',
+        convert_uuid=True,
     )
 
     # use created series to filter dataframe
@@ -53,11 +50,13 @@ def test_is_first_session():
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac311'), True],
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac312'), True]
         ],
-        order_by='event_id'
+        order_by='event_id',
+        convert_uuid=True,
     )
 
-def test_is_new_user():
-    df, modelhub = get_objectiv_dataframe_test(time_aggregation='YYYY-MM-DD')
+
+def test_is_new_user(db_params):
+    df, modelhub = get_objectiv_dataframe_test(db_params, time_aggregation='%Y-%m-%d')
 
     s = modelhub.map.is_new_user(df)
 
@@ -78,7 +77,8 @@ def test_is_new_user():
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac311'), True],
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac312'), True]
         ],
-        order_by='event_id'
+        order_by='event_id',
+        convert_uuid=True,
     )
 
     # use created series to filter dataframe
@@ -96,10 +96,11 @@ def test_is_new_user():
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac311'), True],
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac312'), True]
         ],
-        order_by='event_id'
+        order_by='event_id',
+        convert_uuid=True,
     )
 
-    s = modelhub.map.is_new_user(df, time_aggregation='YYYY-MM')
+    s = modelhub.map.is_new_user(df, time_aggregation='%Y-%m')
 
     assert_equals_data(
         s,
@@ -118,16 +119,141 @@ def test_is_new_user():
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac311'), True],
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac312'), True]
         ],
-        order_by='event_id'
+        order_by='event_id',
+        convert_uuid=True,
     )
 
-def test_is_conversion_event():
-    df, modelhub = get_objectiv_dataframe_test(time_aggregation='YYYY-MM-DD')
+
+@pytest.mark.skip_bigquery
+def test_add_conversion_event(db_params):
+    df, modelhub = get_objectiv_dataframe_test(db_params, time_aggregation='%Y-%m-%d')
+
+    location_stack = df.location_stack.json[{'_type': 'LinkContext', 'id': 'cta-repo-button'}:]
+    event_type = 'ClickEvent'
+    conversion = 'github_clicks'
+    modelhub.add_conversion_event(location_stack=location_stack,
+                                  event_type=event_type,
+                                  name=conversion)
+
+    assert isinstance(modelhub._conversion_events, dict)
+    assert len(modelhub._conversion_events) == 1
+    assert modelhub._conversion_events[conversion] == (location_stack, event_type)
+
+    ser = modelhub.map.is_conversion_event(df, 'github_clicks')
+    assert_equals_data(
+        ser,
+        expected_columns=['event_id', 'is_conversion_event'],
+        expected_data=[
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac301'), False],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac302'), False],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac303'), True],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac304'), False],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac305'), False],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac306'), False],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac307'), False],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac308'), False],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac309'), False],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac310'), False],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac311'), False],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac312'), False]
+        ],
+        order_by='event_id',
+        convert_uuid=True,
+    )
+
+    # location_stack not set
+    modelhub.add_conversion_event(event_type=event_type, name=conversion)
+    assert modelhub._conversion_events[conversion] == (None, event_type)
+
+    ser = modelhub.map.is_conversion_event(df, 'github_clicks')
+
+    assert_equals_data(
+        ser,
+        expected_columns=['event_id', 'is_conversion_event'],
+        expected_data=[
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac301'), True],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac302'), True],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac303'), True],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac304'), True],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac305'), True],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac306'), True],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac307'), True],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac308'), True],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac309'), True],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac310'), True],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac311'), True],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac312'), True]
+        ],
+        order_by='event_id',
+        convert_uuid=True,
+    )
+
+    # event_type not set
+    df, modelhub = get_objectiv_dataframe_test(db_params, time_aggregation='%Y-%m-%d')
+    modelhub.add_conversion_event(location_stack=location_stack, name='github_clicks')
+    assert len(modelhub._conversion_events) == 1
+    assert modelhub._conversion_events[conversion] == (location_stack, None)
+
+    ser = modelhub.map.is_conversion_event(df, 'github_clicks')
+
+    assert_equals_data(
+        ser,
+        expected_columns=['event_id', 'is_conversion_event'],
+        expected_data=[
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac301'), False],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac302'), False],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac303'), True],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac304'), False],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac305'), False],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac306'), False],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac307'), False],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac308'), False],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac309'), False],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac310'), False],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac311'), False],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac312'), False]
+        ],
+        order_by='event_id',
+        convert_uuid=True,
+    )
+
+    # name not set
+    modelhub.add_conversion_event(event_type='ClickEvent')
+    assert len(modelhub._conversion_events) == 2
+    assert modelhub._conversion_events['conversion_2'] == (None, event_type)
+
+    ser = modelhub.map.is_conversion_event(df, 'conversion_2')
+
+    assert_equals_data(
+        ser,
+        expected_columns=['event_id', 'is_conversion_event'],
+        expected_data=[
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac301'), True],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac302'), True],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac303'), True],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac304'), True],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac305'), True],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac306'), True],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac307'), True],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac308'), True],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac309'), True],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac310'), True],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac311'), True],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac312'), True]
+        ],
+        order_by='event_id',
+        convert_uuid=True,
+    )
+
+
+@pytest.mark.skip_bigquery
+def test_is_conversion_event(db_params): # TODO: Remove when bach supports json slicing for BigQuery
+    df, modelhub = get_objectiv_dataframe_test(db_params, time_aggregation='%Y-%m-%d')
 
     # add conversion event
-    modelhub.add_conversion_event(location_stack=df.location_stack.json[{'_type': 'LinkContext', 'id': 'cta-repo-button'}:],
-                            event_type='ClickEvent',
-                            name='github_clicks')
+    modelhub.add_conversion_event(location_stack=df.location_stack.json[{'_type': 'LinkContext',
+                                                                         'id': 'cta-repo-button'}:],
+                                  event_type='ClickEvent', name='github_clicks')
     s = modelhub.map.is_conversion_event(df, 'github_clicks')
 
     assert_equals_data(
@@ -147,7 +273,8 @@ def test_is_conversion_event():
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac311'), False],
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac312'), False]
         ],
-        order_by='event_id'
+        order_by='event_id',
+        convert_uuid=True,
     )
 
     # use created series to filter dataframe
@@ -159,12 +286,21 @@ def test_is_conversion_event():
         expected_data=[
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac303'), True]
         ],
-        order_by='event_id'
+        order_by='event_id',
+        convert_uuid=True,
     )
 
+    # wrong conversion_event name
+    with pytest.raises(KeyError, match="not labeled as a conversion"):
+        modelhub.map.is_conversion_event(df, 'some_clicks')
 
-def test_conversions_counter():
-    df, modelhub = get_objectiv_dataframe_test(time_aggregation='YYYY-MM-DD')
+    with pytest.raises(KeyError, match="not labeled as a conversion"):
+        modelhub.map.is_conversion_event(df, None)
+
+
+@pytest.mark.skip_bigquery  # TODO: Remove when bach supports json slicing for BigQuery
+def test_conversions_counter(db_params):
+    df, modelhub = get_objectiv_dataframe_test(db_params, time_aggregation='%Y-%m-%d')
 
     # add conversion event
     modelhub.add_conversion_event(location_stack=df.location_stack.json[{'_type': 'LinkContext', 'id': 'cta-repo-button'}:],
@@ -190,7 +326,8 @@ def test_conversions_counter():
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac311'), 0],
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac312'), 0]
         ],
-        order_by='event_id'
+        order_by='event_id',
+        convert_uuid=True,
     )
 
     # use created series to filter dataframe
@@ -204,7 +341,8 @@ def test_conversions_counter():
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac302'), 1],
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac303'), 1]
         ],
-        order_by='event_id'
+        order_by='event_id',
+        convert_uuid=True,
     )
 
     s = modelhub.map.conversions_counter(df, 'github_clicks', partition='user_id')
@@ -226,12 +364,14 @@ def test_conversions_counter():
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac311'), 1],
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac312'), 1]
         ],
-        order_by='event_id'
+        order_by='event_id',
+        convert_uuid=True,
     )
 
 
-def test_conversions_in_time():
-    df, modelhub = get_objectiv_dataframe_test(time_aggregation='YYYY-MM-DD')
+@pytest.mark.skip_bigquery  # TODO: Remove when bach supports json slicing for BigQuery
+def test_conversions_in_time(db_params):
+    df, modelhub = get_objectiv_dataframe_test(db_params, time_aggregation='%Y-%m-%d')
 
     # add conversion event
     modelhub.add_conversion_event(
@@ -257,7 +397,8 @@ def test_conversions_in_time():
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac311'), 0],
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac312'), 0]
         ],
-        order_by='event_id'
+        order_by='event_id',
+        convert_uuid=True,
     )
 
     # use created series to filter dataframe
@@ -269,12 +410,14 @@ def test_conversions_in_time():
         expected_data=[
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac303'), True]
         ],
-        order_by='event_id'
+        order_by='event_id',
+        convert_uuid=True,
     )
 
 
-def test_pre_conversion_hit_number():
-    df, modelhub = get_objectiv_dataframe_test(time_aggregation='YYYY-MM-DD')
+@pytest.mark.skip_bigquery  # TODO: Remove when bach supports json slicing for BigQuery
+def test_pre_conversion_hit_number(db_params):
+    df, modelhub = get_objectiv_dataframe_test(db_params, time_aggregation='%Y-%m-%d')
 
     # add conversion event
     modelhub.add_conversion_event(location_stack=df.location_stack.json[{'_type': 'LinkContext', 'id': 'cta-repo-button'}:],
@@ -299,7 +442,8 @@ def test_pre_conversion_hit_number():
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac311'), None],
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac312'), None]
         ],
-        order_by='event_id'
+        order_by='event_id',
+        convert_uuid=True,
     )
 
     # use created series to filter dataframe
@@ -311,7 +455,8 @@ def test_pre_conversion_hit_number():
         expected_data=[
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac301'), True],
         ],
-        order_by='event_id'
+        order_by='event_id',
+        convert_uuid=True,
     )
 
     s = modelhub.map.pre_conversion_hit_number(df, 'github_clicks', partition='user_id')
@@ -333,34 +478,37 @@ def test_pre_conversion_hit_number():
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac311'), 4],
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac312'), 3]
         ],
-        order_by='event_id'
+        order_by='event_id',
+        convert_uuid=True,
     )
 
-def test_time_agg():
-    df, modelhub = get_objectiv_dataframe_test()
+
+def test_time_agg(db_params):
+    df, modelhub = get_objectiv_dataframe_test(db_params)
     s = modelhub.time_agg(df)
 
     assert_equals_data(
         s,
         expected_columns=['event_id', 'time_aggregation'],
         expected_data=[
-            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac301'), '2021-11-30 10:23:36.287'],
-            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac302'), '2021-11-30 10:23:36.290'],
-            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac303'), '2021-11-30 10:23:36.291'],
-            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac304'), '2021-11-30 10:23:36.267'],
-            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac305'), '2021-12-01 10:23:36.276'],
-            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac306'), '2021-12-01 10:23:36.279'],
-            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac307'), '2021-12-02 10:23:36.281'],
-            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac308'), '2021-12-02 10:23:36.281'],
-            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac309'), '2021-12-02 14:23:36.282'],
-            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac310'), '2021-12-03 10:23:36.283'],
-            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac311'), '2021-11-29 10:23:36.286'],
-            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac312'), '2021-11-29 10:23:36.287']
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac301'), '2021-11-30 10:23:36.287000'],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac302'), '2021-11-30 10:23:36.290000'],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac303'), '2021-11-30 10:23:36.291000'],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac304'), '2021-11-30 10:23:36.267000'],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac305'), '2021-12-01 10:23:36.276000'],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac306'), '2021-12-01 10:23:36.279000'],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac307'), '2021-12-02 10:23:36.281000'],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac308'), '2021-12-02 10:23:36.281000'],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac309'), '2021-12-02 14:23:36.282000'],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac310'), '2021-12-03 10:23:36.283000'],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac311'), '2021-11-29 10:23:36.286000'],
+            [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac312'), '2021-11-29 10:23:36.287000']
         ],
-        order_by='event_id'
+        order_by='event_id',
+        convert_uuid=True,
     )
 
-    df, modelhub = get_objectiv_dataframe_test(time_aggregation='YYYY-MM')
+    df, modelhub = get_objectiv_dataframe_test(db_params, time_aggregation='%Y-%m')
     s = modelhub.time_agg(df)
 
     assert_equals_data(
@@ -380,11 +528,12 @@ def test_time_agg():
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac311'), '2021-11'],
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac312'), '2021-11']
         ],
-        order_by='event_id'
+        order_by='event_id',
+        convert_uuid=True,
     )
 
-    df, modelhub = get_objectiv_dataframe_test()
-    s = modelhub.time_agg(df, time_aggregation='YYYY-MM-DD')
+    df, modelhub = get_objectiv_dataframe_test(db_params)
+    s = modelhub.time_agg(df, time_aggregation='%Y-%m-%d')
 
     assert_equals_data(
         s,
@@ -402,11 +551,12 @@ def test_time_agg():
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac310'), '2021-12-03'],
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac311'), '2021-11-29'],
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac312'), '2021-11-29']],
-        order_by='event_id'
+        order_by='event_id',
+        convert_uuid=True,
     )
 
-    df, modelhub = get_objectiv_dataframe_test(time_aggregation='YYYY-MM-DD')
-    s = modelhub.time_agg(df, time_aggregation='YYYY')
+    df, modelhub = get_objectiv_dataframe_test(db_params, time_aggregation='%Y-%m-%d')
+    s = modelhub.time_agg(df, time_aggregation='%Y')
 
     assert_equals_data(
         s,
@@ -424,5 +574,6 @@ def test_time_agg():
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac310'), '2021'],
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac311'), '2021'],
             [UUID('12b55ed5-4295-4fc1-bf1f-88d64d1ac312'), '2021']],
-        order_by='event_id'
+        order_by='event_id',
+        convert_uuid=True,
     )
